@@ -10,7 +10,7 @@ use url::Url;
 use tokio::time;
 use tokio::sync::{mpsc, oneshot};
 use serde::{Deserialize, Serialize};
-use serde_with::{serde_as, NoneAsEmptyString, DurationSeconds, DisplayFromStr, SpaceSeparator, StringWithSeparator};
+use serde_with::{serde_as, NoneAsEmptyString, DurationSeconds, DurationMilliSeconds, DisplayFromStr, SpaceSeparator, StringWithSeparator};
 use serde_repr::Deserialize_repr as DeserializeRepr;
 use shakmaty::fen::Fen;
 use shakmaty::uci::Uci;
@@ -126,6 +126,9 @@ pub enum Work {
         depth: Option<u8>,
         #[serde(default)]
         multipv: Option<NonZeroU8>,
+        #[serde_as(as = "Option<DurationMilliSeconds<u64>>")]
+        #[serde(default)]
+        timeout: Option<Duration>,
     },
     #[serde(rename = "move")]
     Move {
@@ -180,14 +183,15 @@ impl fmt::Display for BatchId {
 #[derive(Debug, Copy, Clone, Deserialize)]
 pub struct NodeLimit {
     classical: u64,
-    nnue: u64,
+    #[serde(default)]
+    sf15: Option<u64>,
 }
 
 impl NodeLimit {
     pub fn get(&self, flavor: EvalFlavor) -> u64 {
         match flavor {
             EvalFlavor::Hce => self.classical,
-            EvalFlavor::Nnue => self.nnue,
+            EvalFlavor::Nnue => self.sf15.unwrap_or(1_500_000),
         }
     }
 }
