@@ -2,6 +2,7 @@ use std::fs;
 use std::io;
 use std::cmp::max;
 use std::fmt;
+use std::error::Error;
 use std::io::Write;
 use std::path::PathBuf;
 use std::str::FromStr;
@@ -36,10 +37,14 @@ pub struct Opt {
     #[clap(long, conflicts_with = "conf", global = true)]
     pub no_conf: bool,
 
-    /// Fishnet API key.
+    /// Fishnet key.
     #[clap(long, alias = "apikey", short = 'k', global = true)]
     pub key: Option<Key>,
 
+    /// Fishnet key file.
+    #[clap(long, parse(from_os_str), conflicts_with = "key", global = true)]
+    pub key_file: Option<PathBuf>,
+
     /// Lichess HTTP endpoint.
     #[clap(long, global = true)]
     pub endpoint: Option<Endpoint>,
@@ -89,7 +94,7 @@ impl FromStr for Endpoint {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         let mut url: Url = s.parse()?;
-        if let Some(stripped_path) = url.path().to_owned().strip_suffix("/") {
+        if let Some(stripped_path) = url.path().to_owned().strip_suffix('/') {
             url.set_path(stripped_path);
         }
         Ok(Endpoint { url })
@@ -129,6 +134,8 @@ impl fmt::Display for KeyError {
     }
 }
 
+impl Error for KeyError {}
+
 impl FromStr for Key {
     type Err = KeyError;
 
@@ -257,16 +264,16 @@ impl FromStr for ParsedDuration {
     type Err = ParseIntError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let (s, factor) = if let Some(s) = s.strip_suffix("d") {
+        let (s, factor) = if let Some(s) = s.strip_suffix('d') {
             (s, 1000 * 60 * 60 * 24)
-        } else if let Some(s) = s.strip_suffix("h") {
+        } else if let Some(s) = s.strip_suffix('h') {
             (s, 1000 * 60 * 60)
-        } else if let Some(s) = s.strip_suffix("m") {
+        } else if let Some(s) = s.strip_suffix('m') {
             (s, 1000 * 60)
         } else if let Some(s) = s.strip_suffix("ms") {
             (s, 1)
         } else {
-            (s.strip_suffix("s").unwrap_or(s), 1000)
+            (s.strip_suffix('s').unwrap_or(s), 1000)
         };
         Ok(ParsedDuration(Duration::from_millis(u64::from(s.trim().parse::<u32>()?) * factor)))
     }
@@ -348,6 +355,18 @@ pub async fn parse_and_configure() -> Opt {
         intro();
     }
 
+    // Handle key file.
+    if !is_systemd {
+        if let Some(key_file) = opt.key_file.take() {
+            opt.key = Some(
+                fs::read_to_string(key_file)
+                    .expect("read key file")
+                    .trim()
+                    .parse()
+                    .expect("valid key from key file"));
+        }
+    }
+
     // Handle config file.
     if opt.command == Some(Command::Configure) || (opt.command != Some(Command::License) && !opt.no_conf) {
         let mut ini = Ini::new();
@@ -400,7 +419,7 @@ pub async fn parse_and_configure() -> Opt {
                     } else {
                         break;
                     }
-                } else if let Some(key) = key.strip_suffix("!") {
+                } else if let Some(key) = key.strip_suffix('!') {
                     (key, false)
                 } else {
                     (key, true)
