@@ -232,8 +232,8 @@ async fn worker(i: usize, assets: Arc<Assets>, tx: mpsc::Sender<Pull>, logger: L
     };
     let mut engine_backoff = RandomizedBackoff::default();
 
-    let max_budget = Duration::from_secs(80);
-    let mut budget = max_budget;
+    let default_budget = Duration::from_secs(60);
+    let mut budget = default_budget;
 
     loop {
         let response = if let Some(job) = job.take() {
@@ -256,7 +256,7 @@ async fn worker(i: usize, assets: Arc<Assets>, tx: mpsc::Sender<Pull>, logger: L
                 }
 
                 // Reset budget, start engine and spawn actor.
-                budget = max_budget;
+                budget = default_budget;
                 let (sf, sf_actor) = stockfish::channel(assets.stockfish.get(flavor).clone(), StockfishInit {
                     nnue: assets.nnue.clone(),
                 }, logger.clone());
@@ -266,12 +266,13 @@ async fn worker(i: usize, assets: Arc<Assets>, tx: mpsc::Sender<Pull>, logger: L
                 (sf, join_handle)
             };
 
-            // Heuristic for timeout. Compare to
+            // Timeout. Compare to
             // https://github.com/ornicar/lila/blob/master/modules/fishnet/src/main/Cleaner.scala
-            budget = min(max_budget, budget + match job.work {
-                Work::Analysis { nodes, .. } => Duration::from_millis(nodes.get(flavor.eval_flavor()) / (2_100_000 / 8000)),
+            let timeout = match job.work {
+                Work::Analysis { timeout, .. } => timeout.unwrap_or_else(|| Duration::from_secs(7)),
                 Work::Move { .. } => Duration::from_secs(2),
-            });
+            };
+            budget = min(default_budget, budget) + timeout;
 
             // Analyse or play.
             let timer = Instant::now();
@@ -311,7 +312,7 @@ async fn worker(i: usize, assets: Arc<Assets>, tx: mpsc::Sender<Pull>, logger: L
 
             // Update budget.
             budget = budget.checked_sub(timer.elapsed()).unwrap_or_default();
-            if budget + Duration::from_secs(8) < max_budget {
+            if budget < default_budget {
                 logger.debug(&format!("Low engine timeout budget: {:?}", budget));
             }
 
